;;; otter-compile.el
;;; This file is part of otter-mode
;;; A major mode for editing and running Otter files.

;;; Author: Holger Schauer <Holger.Schauer@gmd.de>

;;; This file provides the commands to compile 
;;; input files through otter.
;;; Future enhancements might involve searching the output
;;; for a proof etc.

;;; compiling otter files starts an synchronous process
;; so that the complete output of otter can be saved
;; to a file 

(provide 'otter-compile)
;; help fuction: cuts off an extension from a filename
;; if it isn't contained in file_name return file_name
(defun cut-extension (file_name extension)
  (let* ((index (string-match extension file_name)))
    (if (equal index 'nil)
	file_name
	(substring file_name 0 index))))

; this is the main routine for compiling
(defun otter-compile-file (file save-erg-flag)
"Compiles file and puts the output of Otter to the otter-buffer, usually
 *otter* or the name of the input-file extended with *.oout. You can
 scan through the output with M-C-v."
  (interactive)
  (cond ((eql otter-program 'nil)
	 (error 
	  "The variable otter-program is set to nil. 
           Set to absolute path where otter resides.")))
  (if (get-buffer otter-buffer) ; buffer exists already
	 (let* ((actual-buffer-is (buffer-name)))
	   (switch-to-buffer otter-buffer)
	   (erase-buffer)
	   (switch-to-buffer actual-buffer-is))
         (get-buffer-create otter-buffer))
  ; now start a synchronous process running Otter
  ; can't use comint here because otherwise we couldn't save
  ; the output
  (call-process otter-program file otter-buffer t)
  (let* ((actual-buffer-is (buffer-name)))
    (switch-to-buffer-other-window otter-buffer)
    (cond ((equal save-erg-flag 'true) ; if flag is set save buffer content
	 (let* ((cutted-name nil)
		(out-file nil))
	   (setq cutted-name (cut-extension file otter-in-extension1))
	   (cond ((equal cutted-name file) 
		;dann hat das Abschneiden nicht geklappt
		  (setq cutted-name (cut-extension file otter-in-extension2))))
	        ;falls es jetzt wieder nicht klappt, haenge
                ;die Output-Extension einfach so an
	   (setq out-file (concat cutted-name otter-out-extension))
	   (cd (file-name-directory (expand-file-name out-file)))
	   (write-file (expand-file-name out-file))))) 
	   ; implicit (rename-buffer out-file) !!
    (setq mode-name "OTTER Output")
    (setq major-mode 'otter-mode)
    (font-lock-fontify-buffer) ;; enable highlighting :-)
    (setq otter-buffer (buffer-name))
    (beginning-of-buffer) ;; sinnvoll, da am Ende immer Statistiken
    (switch-to-buffer-other-window actual-buffer-is)
   ) ; let zu
)


;; okay, now let's compile a region
;; this works like before - but we have to throw in a region
;; instead of a file to call-process
;; also note that we're not working on a file - and as such
;; saving the output of otter to a file wouldn't make much sense
(defun otter-compile-region ()
  "Compiles the selected region and shows the output of Otter. The
 output of otter will just be displayed in the buffer *otter*."
  (interactive)
  ;; passiert alles nur, wenn die Region echt existiert
;  (cond ((region-exists-p)
    (setq otter-buffer "*otter*")	 
    ;; now clean up otter output buffer if buffer already exists 
    (if (get-buffer otter-buffer)	
	 (let* ((actual-buffer-is (buffer-name)))
	      (switch-to-buffer otter-buffer)
	      (erase-buffer)
	      (switch-to-buffer actual-buffer-is))
	 (get-buffer-create otter-buffer))
    
     (let ((reg-start (region-beginning))
	   (reg-end (region-end))
	   (actual-buffer-is (buffer-name)))
      ; now start a synchronous process running Otter
      (call-process-region reg-start reg-end otter-program nil otter-buffer t)
    
      ;; und jetzt ein bisschen verschoenern :->
      (switch-to-buffer-other-window otter-buffer)
      (setq mode-name "OTTER Output")
      (setq major-mode 'otter-mode)
      (font-lock-fontify-buffer);; enable highlighting :-)
      (setq otter-buffer (buffer-name))
      (beginning-of-buffer);; sinnvoll, da am Ende immer Statistiken
      (switch-to-buffer-other-window actual-buffer-is)) ; let zu
;     )) ;; cond zu
)

; this is easy :-) just get the buffer name and compile the file
(defun otter-compile-buffer ()
"Compiles the active buffer."
(interactive)
(cond ; falls Buffer modifiziert wurde, speichern ? 
  ((and 
    (buffer-modified-p)
    (equal "y" 
     (completing-read "Save buffer ? (y or n) " '(("y" 1) ("n")) nil t "y")))
   (save-buffer)))
(setq otter-input-buffer (buffer-name))
(let* ((file buffer-file-name))
  (otter-compile-file file 'nil)))
       
; this is easy :-) just get the buffer name and compile the file
(defun otter-compile-buffer-save ()
"Compiles the active buffer."
(interactive)
(cond ; falls Buffer modifiziert wurde, speichern ? 
  ((and 
    (buffer-modified-p)
    (equal "y" 
     (completing-read "Save buffer ? (y or n) " '(("y" 1) ("n")) nil t "y")))
   (save-buffer)))
(setq otter-input-buffer (buffer-name))
(let* ((file buffer-file-name))
  (setq file (expand-file-name file))   
  (cd (file-name-directory file))
  (otter-compile-file file 'true)))


; ain't that tricky - just read file-name via minibuffer and
; visit the file. Compile then through otter-compile-file
(defun otter-compile-other-file ()
 "Asks for a file-name and automatically loads and compiles it."
   (interactive)
   (let* ((file (read-file-name "Compile file:")))
     (setq file (expand-file-name file))
     (find-file file)
     (setq otter-input-buffer (buffer-name))
     (setq otter-buffer "*otter*")
     (otter-compile-file file 'nil))
)

; dito - just added the flag for saving when calling otter-compile-file
(defun otter-compile-other-file-save ()
 "Asks for a file-name and automatically loads and compiles it."
   (interactive)
   (let* ((file (read-file-name "Compile file:")))
     (setq file (expand-file-name file))
     (setq otter-buffer "*otter*")
     (find-file file)
     (setq otter-input-buffer (buffer-name))
     (cd (file-name-directory file))
     (otter-compile-file file 'true))
)

